/****************************************************************************
  
  GLUI User Interface Toolkit
  ---------------------------

     glui_panel.cpp - GLUI_Panel control class


          --------------------------------------------------
  Created by Vaughn Clayton, based on glui_rollout
  Copyright (c) 1998 Paul Rademacher

  This program is freely distributable without licensing fees and is
  provided without guarantee or warrantee expressed or implied. This
  program is -not- in the public domain.

*****************************************************************************/

#include "glui.h"
#include "stdinc.h"

/****************************** GLUI_Dropdown::open() **********/

void    GLUI_Dropdown::open( void )
{
  int orig;

  if ( NOT glui )
    return;

  if ( is_open )
    return;

  is_open = true;
  list->hidden=false;
  list->slider->hidden=false;

  orig = set_to_glut_window();

  child_head = collapsed_node.child_head;
  child_tail = collapsed_node.child_tail;

  collapsed_node.child_head = NULL;
  collapsed_node.child_tail = NULL;

  if ( child_head != NULL ) {
    ((GLUI_Control*) child_head)->unhide_internal( true );
  }

  glui->refresh();

  restore_window(orig);
}


/****************************** GLUI_Dropdown::close() **********/

void    GLUI_Dropdown::close( void )
{
  int orig;

  if ( NOT glui )
    return;

  if ( NOT is_open )
    return;

  orig = set_to_glut_window();

  if ( child_head != NULL ) {
    ((GLUI_Control*) child_head)->hide_internal( true );
  }

  collapsed_node.child_head = first_child();
  collapsed_node.child_tail = last_child();

  child_head = NULL;
  child_tail = NULL;

  restore_window(orig);

  this->h = GLUI_DEFAULT_CONTROL_HEIGHT + 7;

  is_open = false;
  list->hidden=true;
  list->slider->hidden=true;

  glui->refresh();
}


/**************************** GLUI_Dropdown::mouse_down_handler() **********/

int   GLUI_Dropdown::mouse_down_handler( int local_x, int local_y )
{
  if ( local_y - y_abs < 18 ) 
  {
    if( is_open )
	  close();
	else
      open();
  }

  currently_inside = true;
  initially_inside = true;

//  draw_pressed();

  if( is_open )
    list->mouse_down_handler(local_x-2, local_y);

  return false;
}


/**************************** GLUI_Dropdown::mouse_up_handler() **********/

int   GLUI_Dropdown::mouse_up_handler( int local_x, int local_y, int inside )
{
  if( is_open )
  {
    if( local_y - y_abs > 18 )
	{
      list->mouse_up_handler(local_x, local_y, inside);
	  if( local_x - (x_abs + w - 16) < 0 ) close();
	}
  }

  int_val = list->int_val;
  strcpy( curr_text.string, list->curr_text.string);
  output_live(true);
  draw_unpressed();

  return false;
}

/******************************* GLUI_Dropdown::mouse_held_down_handler() ***/
int GLUI_Dropdown::mouse_held_down_handler(int local_x, int local_y, int inside)
{
  if( is_open )
    list->mouse_held_down_handler( local_x, local_y, inside );
  return false;
}




/********************************* GLUI_Dropdown::draw() ***********/

void   GLUI_Dropdown::draw( int x, int y )
{
  int orig;
  int name_x, text_x_offset=0;

  if ( NOT can_draw() )
    return;

  orig = set_to_glut_window();
  
//  printf("Dropdown draw: x_abs, y_abs = %d, %d\n", x_abs, y_abs);
	
  name_x = text_x_offset - string_width(this->name) - 3;
  draw_name( name_x, 13 );
  draw_box_inwards_outline( text_x_offset, w, 0, GLUI_DEFAULT_CONTROL_HEIGHT+7);

  if ( NOT active ) {
    draw_box( text_x_offset+3, w-2, 2, 18, 1.0, 1.0, 1.0 );
    if ( NOT enabled )
      glColor3b( 32, 32, 32 );
    else
      glColor3f( 0.0, 0.0, 0.0 );
    glRasterPos2i( text_x_offset+5, 13 );
    draw_string( curr_text.string );
  }
  else {
    draw_box( text_x_offset+3, w-2, 2, 18, .0, .0, .6 );
    glColor3f( 1.0, 1.0, 1.0 );
    glRasterPos2i( text_x_offset+5, 13 );
    draw_string( curr_text.string );
  }


  if ( enabled ) {
    glui->std_bitmaps.
      draw(GLUI_STDBITMAP_DOWN_ARROW,
	   w-glui->std_bitmaps.bitmaps[GLUI_STDBITMAP_DOWN_ARROW].w-1,
	   2 );
  }
  else {
    glui->std_bitmaps.
      draw(GLUI_STDBITMAP_DOWN_ARROW_DIS,
	   w-glui->std_bitmaps.bitmaps[GLUI_STDBITMAP_DOWN_ARROW].w-1,
	   2 );
  }

  if ( is_open )
  {
    list->x_abs = x_abs + 2;
    list->y_abs = y_abs + 20;
    glTranslatef(2, 20, 0);
    list->draw(0, 0);
  }

  restore_window( orig );
}


/***************************** GLUI_Dropdown::update_size() **********/

void   GLUI_Dropdown::update_size( void )
{
//  int text_size;

  if ( NOT glui )
    return;

//  text_size = string_width(name);

  list->update_size();
  h = 22;
  if( is_open ) h += list->h;
  w = list->w;
}


/**************************** GLUI_Dropdown::draw_pressed() ***********/

void   GLUI_Dropdown::draw_pressed( void )
{
  int state, orig;
  int left, right, top, bottom;

  left   = 5;
  right  = w-left;
  top    = 3;
  bottom = 3+16;

  if ( NOT can_draw() )
    return;

  orig  = set_to_glut_window();
  state = glui->set_front_draw_buffer();
  
  glColor3f( 0.0, 0.0, 0.0 );
  glPushMatrix();
  translate_to_origin();

  glBegin( GL_LINE_LOOP );
  glVertex2i( left, top );         glVertex2i( right, top );
  glVertex2i( right, bottom );     glVertex2i( left,bottom );
  glEnd();

  glBegin( GL_LINE_LOOP );
  glVertex2i( left+1, top+1 );         glVertex2i( right-1, top+1 );
  glVertex2i( right-1, bottom-1 );     glVertex2i( left+1,bottom-1 );
  glEnd();

  glPopMatrix();

  glui->restore_draw_buffer(state);
  restore_window(orig);

}


/**************************** GLUI_Dropdown::draw_unpressed() ***********/

void   GLUI_Dropdown::draw_unpressed( void )
{
  if ( NOT can_draw() )
    return;

  translate_and_draw_front();
}


/**************************** GULI_Dropdown::special_handler() ************/
int GLUI_Dropdown::special_handler( int key, int modifiers )
{
  GLUI_Picklist_Item *node, *new_node;

  node     = list->get_item_ptr( list->int_val );
  new_node = NULL;

  if ( key == GLUT_KEY_DOWN ) {
    new_node = (GLUI_Picklist_Item*) node->next();
  }
  else if ( key == GLUT_KEY_UP ) {
    new_node = (GLUI_Picklist_Item*) node->prev();
  }
  else if ( key == GLUT_KEY_HOME ) {
    new_node = (GLUI_Picklist_Item*) list->items_list.first_child();
  }
  else if ( key == GLUT_KEY_END ) {
    new_node = (GLUI_Picklist_Item*) list->items_list.last_child();
  }

  if ( new_node != NULL AND new_node != node ) {
    node = new_node;
    list->set_int_val( node->id );
	int_val = list->int_val;
    strcpy( curr_text.string, list->curr_text.string);
    draw_unpressed();
    output_live(true);
    execute_callback();
    return true;
  }
  else {
    return false;
  }
}


void GLUI_Dropdown::dropdown_callback( int id )
{
//  strcpy( curr_text.string, item->text.string );
//  strcpy( this->curr_text.string, "Picked!");
}


int GLUI_Dropdown::add_item(int id, char *new_text )
{
  list->add_item(id, new_text);
  int_val = list->int_val;
  strcpy( curr_text.string, list->curr_text.string);
  output_live(true);
  update_size();
  return false;
}

