/****************************************************************************
  
  GLUI User Interface Toolkit
  ---------------------------

     glui_picklist - GLUI_Picklist control class


          --------------------------------------------------
  Created by Vaughn Clayton from glui_listbox
  Copyright (c) 1998 Paul Rademacher

  This program is freely distributable without licensing fees and is
  provided without guarantee or warrantee expressed or implied. This
  program is -not- in the public domain.

*****************************************************************************/

#include "glui.h"
#include "stdinc.h"

/****************************** GLUI_Picklist::mouse_down_handler() **********/

int    GLUI_Picklist::mouse_down_handler( int local_x, int local_y )
{
  GLUI_Picklist_Item *item;
  int offset, old_val;

  local_x -= x_abs + 2;
  local_y -= y_abs + 2;
  
  if( local_x >= w - slider->w )
  {
    old_val = scroll_y;
    slider->mouse_down_handler(local_x+x_abs, local_y+y_abs);
	if( scroll_y != old_val ) translate_and_draw_front();
	return false;
  }


  offset=0;
  item = (GLUI_Picklist_Item*) items_list.first_child();
  while( item != NULL AND offset < scroll_y )
  {
    item = (GLUI_Picklist_Item*) item->next();
	offset++;
  }
  offset=0;
  while( item != NULL ) {
    if ( local_y >= offset AND local_y < offset + 15) {
	  old_val = int_val;
	  set_int_val(item->id);
	  
      if ( old_val != int_val) execute_callback();
	  return true;
	}
    offset+=15;
    item = (GLUI_Picklist_Item*) item->next();
  }
  
  return false;
}


/****************************** GLUI_Picklist::mouse_up_handler() **********/

int    GLUI_Picklist::mouse_up_handler( int local_x, int local_y, int inside )
{
  local_x -= x_abs+2;
  local_y -= y_abs+2;
  if( local_x >= w - slider->w )
  {
    slider->mouse_up_handler(local_x+x_abs, local_y+y_abs, inside);
  }

  return false;
}


/****************************** GLUI_Picklist::mouse_held_down_handler() ******/

int    GLUI_Picklist::mouse_held_down_handler( int local_x, int local_y,
					      int inside)
{
  GLUI_Picklist_Item *item;
  int offset, old_val, item_num;

  if ( NOT inside ) return false;

  local_x -= x_abs + 2;
  local_y -= y_abs + 2;
  if( local_x >= w - slider->w )
  {
    old_val = scroll_y;
    slider->mouse_held_down_handler(local_x+x_abs-2, local_y+y_abs-2, inside);
	if( scroll_y != old_val ) translate_and_draw_front();
	return false;
  }


  item_num=0;
  item = (GLUI_Picklist_Item*) items_list.first_child();
  while( item != NULL AND item_num < scroll_y )
  {
    item = (GLUI_Picklist_Item*) item->next();
	item_num++;
  }
  offset=0;
  item_num=0;
  while( item != NULL AND item_num < size) {
    if ( local_y >= offset AND local_y < offset + 15) {
	  old_val = int_val;
	  set_int_val(item->id);
	  
      if ( old_val != int_val) execute_callback();
	  return true;
	}
    offset+=15;
    item = (GLUI_Picklist_Item*) item->next();
	item_num++;
  }
    
  return false;
}


/****************************** GLUI_Picklist::key_handler() **********/

int    GLUI_Picklist::key_handler( unsigned char key,int modifiers )
{
  return false;
}


/****************************** GLUI_Picklist::draw() **********/

void    GLUI_Picklist::draw( int x, int y )
{
  int orig, offset, itemnum;
  GLUI_Picklist_Item *item=NULL;

  if ( NOT can_draw() )
    return;

  orig = set_to_glut_window();

//  printf("Picklist draw: x_abs, y_abs = %d, %d\n", x_abs, y_abs);

  slider->x_abs = x_abs + w - slider->w;
  slider->y_abs = y_abs;
  w -= slider->w - 1; // Only temporarily

  // Draw embossed outline
  draw_box( 0, w, 0, h, 1.0, 1.0, 1.0 );

  glLineWidth( 1.0 );
  glColor3f( 1.0, 1.0, 1.0 );
  glBegin( GL_LINE_LOOP );
  glVertex2i( 0, 0 );    glVertex2i( w, 0 );
  glVertex2i( w, h );    glVertex2i( 0, h );

  glVertex2i( 1, 1 );    glVertex2i( w-1, 1 );
  glVertex2i( w-1, h-1 );    glVertex2i( 1, h-1 );
  glEnd();
    
  glColor3f( .5, .5, .5 );
  glBegin( GL_LINE_LOOP );
  glVertex2i( 0, 0 );
  glVertex2i( w-1, 0 );
  glVertex2i( w-1, h-1 );
  glVertex2i( 0, h-1 );
  glEnd();

	
  glLineWidth( 1.0 );



  item = (GLUI_Picklist_Item*) items_list.first_child();
  itemnum=0;
  while( (item!=NULL) && (itemnum < scroll_y) )
  {
    item = (GLUI_Picklist_Item*) item->next();
	itemnum++;
  }
  
  offset=0;
  itemnum=0;
  glColor3f( .0, .0, .0 );
  while( (item != NULL) && (itemnum < size) ) {
    glColor3f( .0, .0, .0 );
    if ( int_val == item->id ) {
	
	  glColor3f( 1.0, 1.0, 1.0);
      if ( NOT active ) {
        glEnable( GL_LINE_STIPPLE );
        glLineStipple( 1, 0x5555 );
    
        glColor3f( 0., 0., 0. );
    
        glBegin( GL_LINE_LOOP );
        glVertex2i( 1, offset );         glVertex2i( w-2, offset );
        glVertex2i( w-2, offset+15 );     glVertex2i( 1, offset+15 );
        glEnd();
    
        glDisable( GL_LINE_STIPPLE );
      }
	  else {
        draw_box(1, w-1, offset, offset+15, 0.0, 0.0, 0.6 );
  	    glColor3f( 1.0, 1.0, 1.0);
	  }
	}
	  
    glRasterPos2i( 3, 12+offset );
    draw_string( item->text.string );
    offset+=15;
    item = (GLUI_Picklist_Item*) item->next();
	itemnum++;
  }
  
  glTranslatef(w-1, 0, 0);
  slider->draw(0,0);
  w += slider->w-1;
  
  restore_window(orig);
  return;
}



void GLUI_Picklist::scrollCB( int arg )
{
//  draw(0, 0);
  translate_and_draw_front();
}








/************************************ GLUI_Picklist::set_size() ********/

void   GLUI_Picklist::set_size( int numitems )
{
  if ( NOT glui )
    return;
	
  scroll_y = 0;
  size = numitems;
  slider->x_abs = 30;
//  slider->set_int_limits(0, numitems-size>0? numitems-size : 0, GLUI_LIMIT_CLAMP);
  update_size();
}






/************************************ GLUI_Picklist::update_size() ********/

void   GLUI_Picklist::update_size( void )
{
//  int text_size;
//  int item_text_size;

  if ( NOT glui )
    return;

//  text_size = string_width( name );

  /*** Find the longest item string ***/
//  item_text_size = 0;

//  if ( text_x_offset < text_size +2 )
//    delta = text_size+2-text_x_offset;

//  text_x_offset += delta;
//  if ( w < text_x_offset+MAX(GLUI_EDITTEXT_MIN_TEXT_WIDTH,item_text_size)+20)
//    w = text_x_offset + MAX( GLUI_EDITTEXT_MIN_TEXT_WIDTH,item_text_size)+20;

  h = 2 + size * 15;
  if( size == 0 ) h += 15;
  if( slider ) {
    slider->set_int_limits(0, item_count-size>0? item_count-size : 0, GLUI_LIMIT_CLAMP);
    slider->h = h;
	slider->update_size();
  }
}



/********************************* GLUI_Picklist::set_int_val() **************/

void    GLUI_Picklist::set_int_val( int new_val )
{
  /*  int_val = new_val;              */
  if(new_val < scroll_y) scroll_y = new_val;
  if(new_val >= scroll_y+size) scroll_y = new_val-size+1;

  do_selection( new_val );
  slider->sync_live(false, true); // No recurse, draw

  /*** Update the variable we're (possibly) pointing to, and update the main gfx ***/
  output_live(true);
}


/**************************** GLUI_Picklist::draw_active_area() **************/

void    GLUI_Picklist::draw_active_area( void )
{
  int orig;

  if ( NOT can_draw() )
    return;

  orig = set_to_glut_window();

  restore_window(orig);
}


/**************************************** GLUI_Picklist::add_item() **********/

int  GLUI_Picklist::add_item( int id, char *new_text )
{
  GLUI_Picklist_Item *new_node = new GLUI_Picklist_Item;
  GLUI_Picklist_Item *head;

  strcpy( new_node->text, new_text );

  new_node->id = id;
  new_node->update_size();

  head = (GLUI_Picklist_Item*) items_list.first_child();
  new_node->link_this_to_parent_last( &items_list );

  if ( head == NULL ) {  // This is first item added

    int_val       = id+1;  /** Different than id **/
    do_selection( id );
    last_live_int = id;

    if( glui )
      glui->post_update_main_gfx();
  }

  scroll_y = 0;
  item_count++;

  if( item_count <= size ) slider->disable();
  else slider->enable();
  slider->update_size();
  
  /*** Check if we need to increase control size ***/
  if ( w < text_x_offset + MAX( GLUI_EDITTEXT_MIN_TEXT_WIDTH, string_width( new_text ) ) + 20 ) {
    w = text_x_offset + MAX( GLUI_EDITTEXT_MIN_TEXT_WIDTH, string_width( new_text ) ) + 20;

    if ( glui )
      glui->refresh();
  }

  if ( glui )
    glui->refresh();

  return true;
}


/************************************** GLUI_Picklist::delete_item() **********/

int  GLUI_Picklist::delete_item( char *text )
{
  GLUI_Picklist_Item *node = get_item_ptr( text );

  scroll_y = 0;
  if ( node ) {
    node->unlink();
    delete node;
	item_count--;
    if( item_count <= size ) slider->disable();
    else slider->enable();
	if( glui ) glui->refresh();
    return true;
  }
  else {
    return false;
  }
}


/************************************** GLUI_Picklist::delete_item() **********/

int  GLUI_Picklist::delete_item( int id )
{
  GLUI_Picklist_Item *node = get_item_ptr( id );

  scroll_y = 0;
  if ( node ) {
    node->unlink();
    delete node;
	item_count--;
    if( item_count <= size ) slider->disable();
    else slider->enable();
	if( glui ) glui->refresh();
    return true;
  }
  else {
    return false;
  }
}


/************************************** GLUI_Picklist::sort_items() **********/

int  GLUI_Picklist::sort_items( void )
{
  return false;
}


/********************************************* GLUI_Picklist::dump() **********/

void     GLUI_Picklist::dump( FILE *output )
{
  GLUI_Picklist_Item *item;

  /*  printf( "%p\n", (char*) name );              */

  fprintf( output, "Picklist: %s\n", (char*) name );

  item = (GLUI_Picklist_Item *) items_list.first_child();
  while( item ) {
    fprintf( output, "         %3d : %s\n", item->id, (char*) item->text );
    
    item = (GLUI_Picklist_Item *) item->next();
  }
}


/************************************ GLUI_Picklist::get_item_ptr() **********/

GLUI_Picklist_Item *GLUI_Picklist::get_item_ptr( char *text )
{
  GLUI_Picklist_Item *item;

  item = (GLUI_Picklist_Item *) items_list.first_child();
  while( item ) {
    if ( NOT strcmp( item->text, text ))
      return item;
    
    item = (GLUI_Picklist_Item *) item->next();
  }

  return NULL;
}


/************************************ GLUI_Picklist::get_item_ptr() **********/

GLUI_Picklist_Item *GLUI_Picklist::get_item_ptr( int id )
{
  GLUI_Picklist_Item *item;

  item = (GLUI_Picklist_Item *) items_list.first_child();
  while( item ) {
    if ( item->id == id )
      return item;
    
    item = (GLUI_Picklist_Item *) item->next();
  }

  return NULL;
}




/****************************** GLUI-Picklist::idle() **********************/
int GLUI_Picklist::needs_idle( void )
{
  return slider->needs_idle();
}

void GLUI_Picklist::idle( void )
{
  int old_val;
  old_val = scroll_y;
  slider->idle();
  if( old_val != scroll_y ) translate_and_draw_front();
}





/************************************ GLUI_Picklist::mouse_over() **********/

static void picklist_callback( int i )
{
  int old_val;

  if ( NOT GLUI_Master.curr_left_button_glut_menu OR 
       GLUI_Master.curr_left_button_glut_menu->type != GLUI_CONTROL_PICKLIST ) 
    return;

  old_val = ((GLUI_Picklist*)GLUI_Master.curr_left_button_glut_menu)->int_val;
  ((GLUI_Picklist*)GLUI_Master.curr_left_button_glut_menu)->set_int_val(i);

  /****   If value changed, execute callback   ****/
  if ( old_val != 
       ((GLUI_Picklist*)GLUI_Master.curr_left_button_glut_menu)->int_val ) {
    ((GLUI_Picklist*)GLUI_Master.curr_left_button_glut_menu)->execute_callback();
  }
}


/*************************************** GLUI_Picklist::mouse_over() **********/

int     GLUI_Picklist::mouse_over( int state, int x, int y )
{
/*
  GLUI_Picklist_Item *item;

  if ( state AND enabled AND x > x_abs + text_x_offset) {
    

    glut_menu_id = glutCreateMenu(picklist_callback);

    item = (GLUI_Picklist_Item *) items_list.first_child();
    while( item ) {
      glutAddMenuEntry( item->text, item->id );
      item = (GLUI_Picklist_Item *) item->next();
    }

    glutAttachMenu( GLUT_LEFT_BUTTON);
    
    GLUI_Master.set_left_button_glut_menu_control( this );
  }
  else if ( glut_menu_id != -1 ) {
    glutDetachMenu( GLUT_LEFT_BUTTON );
    glutDestroyMenu( glut_menu_id );
    glut_menu_id = -1;
  }
*/

  return true;
}


/************************************ GLUI_Picklist::do_selection() **********/

int    GLUI_Picklist::do_selection( int item_num )
{
  GLUI_Picklist_Item *item, *sel_item;

  /***  Is this item already selected?  ***/
  if ( item_num == int_val )
    return false;

  sel_item = NULL;
  item     = (GLUI_Picklist_Item *) items_list.first_child();
  while( item ) {
    if ( item->id == item_num ) {
      sel_item = item;
      break;
    }
    
    item = (GLUI_Picklist_Item *) item->next();
  }

  if ( NOT sel_item )
    return false;

  /*  printf( "-> %s\n", (char*) sel_item->text );              */

  int_val = item_num;
  strcpy( curr_text.string, sel_item->text.string );

  translate_and_draw_front();

  return true;
}


/*********************************** GLUI_Picklist::~GLUI_Picklist() **********/

GLUI_Picklist::~GLUI_Picklist( )
{
  GLUI_Picklist_Item *item, *tmp_item;

  item = (GLUI_Picklist_Item *) items_list.first_child();
  while( item ) {
    tmp_item = item;

    delete item;
    
    item = (GLUI_Picklist_Item *) tmp_item->next();
  }
}


/****************************** GLUI_Picklist::special_handler() **********/

int    GLUI_Picklist::special_handler( int key,int modifiers )
{
  GLUI_Picklist_Item *node, *new_node;

  node     = get_item_ptr( int_val );
  new_node = NULL;

  if ( key == GLUT_KEY_DOWN ) {
    new_node = (GLUI_Picklist_Item*) node->next();
  }
  else if ( key == GLUT_KEY_UP ) {
    new_node = (GLUI_Picklist_Item*) node->prev();
  }
  else if ( key == GLUT_KEY_HOME ) {
    new_node = (GLUI_Picklist_Item*) items_list.first_child();
  }
  else if ( key == GLUT_KEY_END ) {
    new_node = (GLUI_Picklist_Item*) items_list.last_child();
  }

  if ( new_node != NULL AND new_node != node ) {
    node = new_node;
    set_int_val( node->id );
    execute_callback();
    return true;
  }
  else {
    return false;
  }
}


/************************* GLUI_Picklist::increase_width( void ) ***********/

void    GLUI_Picklist::increase_width( void )
{
}
