/****************************************************************************
  
  GLUI User Interface Toolkit
  ---------------------------

     glui_slider.cpp - GLUI_Slider class

  notes: 


          --------------------------------------------------
  Created by Vaughn Clayton from glui_spinner
  Copyright (c) 1998 Paul Rademacher

  This program is freely distributable without licensing fees and is
  provided without guarantee or warrantee expressed or implied. This
  program is -not- in the public domain.

*****************************************************************************/

  
#include "glui.h"
#include "stdinc.h"
#include <math.h>

/*static int __debug=0;              */

#define  GLUI_SLIDER_CALLBACK_INTERVAL    1

 

/****************************** GLUI_Slider::mouse_down_handler() **********/

int    GLUI_Slider::mouse_down_handler( int local_x, int local_y )
{
//    printf( "slider: mouse down  : %d,%d   arrow:%d\n", local_x, local_y,
//      find_arrow( local_x, local_y ));
  last_x = local_x;
  last_y = local_y;
  
  if(state == GLUI_SLIDER_STATE_NONE)
  {
    repeat_count=repeat_delay;
    state = find_arrow( local_x, local_y );
    if ( state != GLUI_SLIDER_STATE_UP AND state != GLUI_SLIDER_STATE_DOWN )
      return true;
  }
  else return true;

  do_click();

  
  return false;
}


/******************************** GLUI_Slider::mouse_up_handler() **********/

int    GLUI_Slider::mouse_up_handler( int local_x, int local_y, int inside )
{
  state = GLUI_SLIDER_STATE_NONE;

  /*  printf("slider: mouse up  : %d/%d    inside: %d\n",local_x,local_y,inside);              */

  /*glutSetCursor( GLUT_CURSOR_INHERIT );              */
  glutSetCursor( GLUT_CURSOR_LEFT_ARROW );

  draw_arrows();

  /*  do_callbacks(); --- stub               */
  /*  if ( callback )               */
  /*  callback( this->user_id );              */
  
  return false;
}


/***************************** GLUI_Slider::mouse_held_down_handler() ******/

int    GLUI_Slider::mouse_held_down_handler( int local_x, int local_y,
					      int new_inside)
{
/*    printf("slider: mouse held: %d/%d    inside: %d\n",local_x,local_y,
      new_inside);*/
      
  if ( state == GLUI_SLIDER_STATE_DRAG )
    do_drag( local_x, local_y );
  return false;
}


/****************************** GLUI_Slider::key_handler() **********/

int    GLUI_Slider::key_handler( unsigned char key,int modifiers )
{
  return true;
}


/****************************** GLUI_Slider::draw() **********/

void    GLUI_Slider::draw( int x, int y )
{
  int orig;
  float offset;

  if ( NOT can_draw() )
    return;
  orig = set_to_glut_window();

//  printf("Slider draw: x_abs, y_abs = %d, %d\n", x_abs, y_abs);
  
  if(orientation==GLUI_SLIDER_HORIZONTAL)
  {
    offset = tab_pos + GLUI_SLIDER_ARROW_WIDTH + 1;
    
    draw_box(GLUI_SLIDER_ARROW_WIDTH+1, w-GLUI_SLIDER_ARROW_WIDTH, GLUI_SLIDER_ARROW_Y, h-2, 0.5, 0.5, 0.5);
    if( enabled ) {
      glui->std_bitmaps.draw( GLUI_STDBITMAP_LEFT_ARROW, 0, GLUI_SLIDER_ARROW_Y);
      glui->std_bitmaps.draw( GLUI_STDBITMAP_RIGHT_ARROW, w-GLUI_SLIDER_ARROW_WIDTH-1, GLUI_SLIDER_ARROW_Y);
      glui->draw_raised_box(offset, GLUI_SLIDER_ARROW_Y+2, tab_size, GLUI_SLIDER_ARROW_HEIGHT-2);
    }
    else {
      glui->std_bitmaps.draw( GLUI_STDBITMAP_LEFT_ARROW_DIS, 0, GLUI_SLIDER_ARROW_Y);
      glui->std_bitmaps.draw( GLUI_STDBITMAP_RIGHT_ARROW_DIS, w-GLUI_SLIDER_ARROW_WIDTH-1, GLUI_SLIDER_ARROW_Y);
    }

    draw_box(offset+1, offset+tab_size-1, GLUI_SLIDER_ARROW_Y+2, GLUI_SLIDER_ARROW_Y+GLUI_SLIDER_ARROW_HEIGHT-2, 0.75, 0.75, 0.75);
  }
  else // orientation==GLUI_SLIDER_VERTICAL
  {
    offset = tab_pos + GLUI_SLIDER_ARROW_HEIGHT + 1;

    draw_box(3, w-1, GLUI_SLIDER_ARROW_HEIGHT, h-GLUI_SLIDER_ARROW_HEIGHT, 0.5, 0.5, 0.5);
    if( enabled ) {
      glui->std_bitmaps.draw( GLUI_STDBITMAP_DOWN_ARROW, 2, h-GLUI_SLIDER_ARROW_HEIGHT-1);
      glui->std_bitmaps.draw( GLUI_STDBITMAP_UP_ARROW, 2, 1);
      glui->draw_raised_box(3, offset, GLUI_SLIDER_ARROW_WIDTH-1, tab_size+1);
    }
    else {
      glui->std_bitmaps.draw( GLUI_STDBITMAP_DOWN_ARROW_DIS, 2, h-GLUI_SLIDER_ARROW_HEIGHT-1);
      glui->std_bitmaps.draw( GLUI_STDBITMAP_UP_ARROW_DIS, 2, 1);
    }

    draw_box(5, w-3, offset+1, offset+tab_size-1, 0.75,0.75,0.75);  
  }


  if ( active ) {
    glColor3ub( 0, 0, 0 );
    glEnable( GL_LINE_STIPPLE );
    glLineStipple( 1, 0x5555 );
    
  glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
  glDisable( GL_CULL_FACE );
  glBegin( GL_QUADS );
  glVertex2i( 0, 0 );
  glVertex2i( w, 0 );
  glVertex2i( w, h );
  glVertex2i( 0, h );
  glEnd();
  glDisable( GL_LINE_STIPPLE );  
  glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
    
  }
  else {
    glColor3ub( glui->bkgd_color.r,glui->bkgd_color.g,glui->bkgd_color.b );
  } 
  if( parent() )
    if( ( (GLUI_Panel*)parent() )->type == GLUI_CONTROL_PICKLIST )
    {
      glColor3ub( glui->bkgd_color.r,glui->bkgd_color.g,glui->bkgd_color.b );
      glDisable( GL_LINE_STIPPLE );
    }



  restore_window( orig );
}





/********************************* GLUI_Slider::special_handler() **********/

int    GLUI_Slider::special_handler( int key,int modifiers )
{
  if ( key == GLUT_KEY_UP ) {    /** Simulate a click in the up arrow **/
    mouse_down_handler( x_abs + w - GLUI_SLIDER_ARROW_WIDTH + 1,
			y_abs + GLUI_SLIDER_ARROW_Y+1 );
    mouse_up_handler( x_abs + w - GLUI_SLIDER_ARROW_WIDTH + 1,
		      y_abs + GLUI_SLIDER_ARROW_Y+1, true );
  }
  else if ( key == GLUT_KEY_DOWN ) {  /** Simulate a click in the up arrow **/
    mouse_down_handler(x_abs + w - GLUI_SLIDER_ARROW_WIDTH + 1,
		       y_abs+GLUI_SLIDER_ARROW_Y+1+GLUI_SLIDER_ARROW_HEIGHT);
    mouse_up_handler( x_abs + w - GLUI_SLIDER_ARROW_WIDTH + 1,
		      y_abs+GLUI_SLIDER_ARROW_Y+1 +GLUI_SLIDER_ARROW_HEIGHT,
		      true );
  }
  else if ( key == GLUT_KEY_HOME ) {  /** Set value to limit top - 
					or increment by 10 **/
  }
  else if ( key == GLUT_KEY_END ) {  
  }

  return true;
}


/******************************* GLUI_Slider::set_float_val() ************/

void   GLUI_Slider::set_float_val( float new_val )
{ 
  if ( has_limits == GLUI_LIMIT_CLAMP ) {
    /*** Clamp the new value to the existing limits ***/

    CLAMP( new_val, float_low, float_high );
  } 
  else if ( has_limits == GLUI_LIMIT_WRAP ) {
    /*** Clamp the value cyclically to the limits - that is, if the
      value exceeds the max, set it the the minimum, and conversely ***/

    if ( new_val < float_low )
      new_val = float_high;
    if ( new_val > float_high )
      new_val = float_low;
  }

  tab_pos = (new_val-float_low) * (float)tab_range / (float_high - float_low);
  float_val = new_val;
  int_val   = (int) new_val;  /* Mirror the value as an int, too */
  
  output_live(true);
}


/********************************** GLUI_Slider::set_int_val() ************/

void   GLUI_Slider::set_int_val( int new_val )
{
  if ( has_limits == GLUI_LIMIT_CLAMP ) {
    /*** Clamp the new value to the existing limits ***/

    CLAMP( new_val, int_low, int_high );
  }
  else if ( has_limits == GLUI_LIMIT_WRAP ) {
    /*** Clamp the value cyclically to the limits - that is, if the
      value exceeds the max, set it the the minimum, and conversely ***/

    if ( new_val < int_low )
      new_val = int_high;
    if ( new_val > int_high )
      new_val = int_low;
  }

  tab_pos = (new_val-int_low) * ((float)tab_range / (float)(int_high-int_low));
  int_val   = new_val;
  float_val = (float) new_val;   /* We mirror the value as a float, too */
  output_live(true);
}


/************************************ GLUI_Slider::update_size() **********/

void   GLUI_Slider::update_size( void )
{
  if( orientation == GLUI_SLIDER_HORIZONTAL ) {
    h = GLUI_SLIDER_ARROW_HEIGHT + 4;
    if( w < GLUI_SLIDER_ARROW_WIDTH*3+3)
      w = GLUI_SLIDER_ARROW_WIDTH*6 + 3;
    tab_range = w - GLUI_SLIDER_ARROW_WIDTH*2 - 2 - tab_size;
  }
  else {
    w = GLUI_SLIDER_ARROW_WIDTH + 4;
    if( h < GLUI_SLIDER_ARROW_HEIGHT*3+3)
      h = GLUI_SLIDER_ARROW_HEIGHT*6 + 3;
    tab_range = h - GLUI_SLIDER_ARROW_HEIGHT*2 - 2 - tab_size;
  }
  
  if( data_type == GLUI_SLIDER_FLOAT )
    tab_pos = (float_val-float_low) * (float)tab_range / (float_high - float_low);
  else
    tab_pos = (int_val-int_low) * ((float)tab_range / (float)(int_high-int_low));
  
}
 

/************************************ GLUI_Slider::find_arrow() ************/

int    GLUI_Slider::find_arrow( int local_x, int local_y )
{
  float offset;

  local_x -= x_abs;
  local_y -= y_abs;
//  printf("Find_arrow Local X,Y: %d, %d tab_size: %d\n", local_x, local_y, tab_size);

  if( orientation == GLUI_SLIDER_HORIZONTAL ) 
  {
    offset = tab_pos + GLUI_SLIDER_ARROW_WIDTH + 1;
    
    if( local_x > 1 AND local_x <= GLUI_SLIDER_ARROW_WIDTH+1 )
      return GLUI_SLIDER_STATE_DOWN;
    if( local_x > (w - GLUI_SLIDER_ARROW_WIDTH) AND local_x <= w )
      return GLUI_SLIDER_STATE_UP;
    if( local_x > offset AND local_x <= (offset+tab_size) )
    {
      last_x = local_x - offset;
      return GLUI_SLIDER_STATE_DRAG;
    }
  }
  else
  {
    offset = tab_pos + GLUI_SLIDER_ARROW_HEIGHT;
    if( local_y > 0 AND local_y <= GLUI_SLIDER_ARROW_HEIGHT )
      return GLUI_SLIDER_STATE_DOWN;
    if( local_y >= (h - GLUI_SLIDER_ARROW_HEIGHT) AND local_y < h )
      return GLUI_SLIDER_STATE_UP;
    if( local_y >= offset AND local_y <= (offset+tab_size+1) )
    {
      last_y = local_y - offset;
      return GLUI_SLIDER_STATE_DRAG;
    }
  }

  return GLUI_SLIDER_STATE_NONE;
}


/****************************** GLUI_Slider::draw_arrows() **********/

void    GLUI_Slider::draw_arrows( void )
{
  if ( NOT can_draw() )
    return;

  translate_and_draw_front();
}

void GLUI_Slider::update_parent( void )
{
  GLUI_Panel *p;
  if ( NOT parent() ) return;
  p = (GLUI_Panel *)parent();

  glMatrixMode( GL_MODELVIEW );
  glPushMatrix();

  glTranslatef( (float) p->x_abs + .5, (float) p->y_abs + .5, 0.0 );
  p->draw(0, 0);

  glPopMatrix();  
}
  

/***************************************** GLUI_Slider::do_click() **********/

void    GLUI_Slider::do_click( void )
{
  int    direction=0;
  float  incr;
  float  modifier_factor;

  if ( state == GLUI_SLIDER_STATE_UP )
    direction = +1;
  else if ( state == GLUI_SLIDER_STATE_DOWN )
    direction = -1;

  modifier_factor = 1.0;
  if ( glui ) {
    if ( glui->curr_modifiers & GLUT_ACTIVE_CTRL )
      modifier_factor = float_high - float_low;
  }

  if ( this->data_type == GLUI_SLIDER_FLOAT ) {
    incr = direction * user_speed * modifier_factor;
    set_float_val( float_val + incr );
  }
  else {
    incr = direction * user_speed * modifier_factor;
    set_int_val( int_val + incr );
  }

  if( can_draw() ) {
    draw_arrows();
    update_parent();
  }
  
  /*** Now update live variable and do callback.  We don't want
    to do the callback on each iteration of this function, just on every 
    i^th iteration, where i is given by GLUI_SLIDER_CALLBACK_INTERVAL ****/
  callback_count++;
  if ( (callback_count % GLUI_SLIDER_CALLBACK_INTERVAL ) == 0 )
    do_callbacks();
}


/***************************************** GLUI_Slider::do_drag() **********/

void    GLUI_Slider::do_drag( int x, int y )
{
  float range, wdth;
  int local_x, local_y;
  int local_w, local_h;
  
  local_x = x - x_abs;
  local_y = y - y_abs;
//  printf("Drag. State: %d, Location: %d, %d (%d, %d)\n", state, x, y, local_x, local_y);
  
  if(data_type==GLUI_SLIDER_FLOAT) range = float_high - float_low;
    else range = int_high - int_low;

  if( orientation == GLUI_SLIDER_HORIZONTAL )
  {
    local_x -= GLUI_SLIDER_ARROW_WIDTH;
    local_w = w - GLUI_SLIDER_ARROW_WIDTH*2;
    wdth = local_w / (range) * user_speed;
    local_w -= wdth;
    tab_pos = local_x - last_x;
  }
  else // orientation == GLUI_SLIDER_VERTICAL
  {
    local_y -= GLUI_SLIDER_ARROW_HEIGHT;
    local_h = h - GLUI_SLIDER_ARROW_HEIGHT*2;
    wdth = local_h / (range) * user_speed;
    local_h -= wdth;
    tab_pos = local_y - last_y;
  }
  CLAMP( tab_pos, 0, (tab_range) );
  
  if( data_type == GLUI_SLIDER_FLOAT ) 
    float_val = float_low + range * (float)tab_pos / (float)(tab_range);
  else 
    int_val = int_low + rint(range * (float)tab_pos / (float)(tab_range));
  output_live(true);

  if ( can_draw() )
  {
    draw_arrows();
    update_parent();
  }

  /*** Now update live variable and do callback.  We don't want
    to do the callback on each iteration of this function, just on every 
    i^th iteration, where i is given by GLUI_SLIDER_CALLBACK_INTERVAL ****/

  callback_count++;
  if ( (callback_count % GLUI_SLIDER_CALLBACK_INTERVAL ) == 0 )
    do_callbacks();
}


/***************************************** GLUI_Slider::needs_idle() ******/

int     GLUI_Slider::needs_idle( void )
{
  if  (state == GLUI_SLIDER_STATE_UP OR state == GLUI_SLIDER_STATE_DOWN )
    return true;
  else
    return false;
}

/***************************************** GLUI_Slider::idle() **********/

void    GLUI_Slider::idle( void )
{
  if ( NOT needs_idle() )
    return;
  else
  {
    if( --repeat_count == 0 )
    {
      do_click();
      repeat_count = repeat_rate;
    }
  }
}


/************************************ GLUI_Slider::do_callbacks() **********/

void    GLUI_Slider::do_callbacks( void )
{
  if ( NOT first_callback ) {
    if ( data_type == GLUI_SLIDER_INT AND int_val == last_int_val ) {
      return;
    }
    
    if ( data_type == GLUI_SLIDER_FLOAT AND float_val == last_float_val ) {
      return;
    }
  }
  
  this->execute_callback();

  last_int_val   = int_val;
  last_float_val = float_val;
  first_callback = false;
}


/********************************* GLUI_Slider::set_float_limits() *********/

void GLUI_Slider::set_float_limits( float low, float high, int limit_type )
{
  has_limits  = limit_type;
  float_low   = low;
  float_high  = high;

  if(high - low <= 0.0) enabled=false;
  
  if ( NOT IN_BOUNDS( float_val, float_low, float_high ))
    set_float_val( float_low );

  int_low     = (int) float_low;
  int_high    = (int) float_high;
  tab_size = GLUI_SLIDER_TAB_SIZE;
}


/*********************************** GLUI_Slider::set_int_limits() *********/

void   GLUI_Slider::set_int_limits( int low, int high, int limit_type )
{
  has_limits  = limit_type;
  int_low     = low;
  int_high    = high;

  if(high - low <= 0)
  {
    if( orientation == GLUI_SLIDER_HORIZONTAL )
      tab_size = w - GLUI_SLIDER_ARROW_WIDTH*2 - 2;    
    else
      tab_size = h - GLUI_SLIDER_ARROW_HEIGHT*2 - 2;    
  }
  else
  {
    if( orientation == GLUI_SLIDER_HORIZONTAL )
      tab_size = (float)(w - GLUI_SLIDER_ARROW_WIDTH*2 - 2) / (float)(high-low + 1);
    else
      tab_size = (float)(h - GLUI_SLIDER_ARROW_HEIGHT*2 - 2) / (float)(high-low + 1);
  }
  
  if( tab_size < GLUI_SLIDER_TAB_SIZE ) tab_size = GLUI_SLIDER_TAB_SIZE;

  if ( NOT IN_BOUNDS( int_val, int_low, int_high ))
    set_int_val( int_low );
    
  update_size();

  float_low   = (float) int_low;
  float_high  = (float) int_high;
}
 


/*************************************** GLUI_Slider:get_text() *************/

char    *GLUI_Slider::get_text( void )
{ 
  return text;
}


/********************************** GLUI_Slider:get_float_val() *************/

float    GLUI_Slider::get_float_val( void )
{
  return float_val;
}


/********************************** GLUI_Slider:get_int_val() *************/

int    GLUI_Slider::get_int_val( void )
{
  return int_val;
}


/******************************** GLUI_Slider::set_orientation() *********/
void GLUI_Slider::set_orientation( int orient )
{
  orientation = orient;
  update_size();
  if( data_type == GLUI_SLIDER_FLOAT )
    set_float_val( float_val );
  else
    set_int_val( int_val );
}
