/**************************************************
*  libsir.h - Header file for libsir.c
*  SIR image file format utility library
*  Author: Vaughn C. Clayton
*  (C) August 2000 Brigham Young University
***************************************************/

#ifndef LIBSIR_H
#define LIBSIR_H

#define SHORT short
#define FLOAT float


struct sirfileheader
{
	SHORT xpixels, ypixels;   // Image size in pixels
	SHORT xdegrees, ydegrees; // Span of degrees (in hundredths of degrees)
	SHORT headertype;         // Old header < 15
	SHORT xscale, yscale;     // Scaling factors
	SHORT olon, olat;  // Origin of longitude (-180<=olon<=180) and lattitude
	SHORT ioff;               // Offset to be added to scaled value
	SHORT iscale;             // Scale factor: ival=(val-ioff)/iscale
	SHORT year;               // Year of data used
	SHORT sday, smin;// Starting Julian Date, time of day for first data (in min)
	SHORT eday, emin;// Ending Julian Date, time of day for last data (in min)
	SHORT proj_type; // Projection type.  See DEFINEs above for types
	SHORT region;             // Region ID code
	SHORT image_type;         // Image type. See DEFINEs above for types
	char sensor[40];     // Name of sensor used to collect data
	SHORT null1;              // Should always == 0
	SHORT num_headers;        // Number of 512 byte header blocks
	SHORT num_desc;           // Number of 512 byte description blocks
	SHORT len_desc;           // Number of bytes in description
	SHORT num_opts;           // Number of optional integers
	SHORT polarization;       // 0=n/a, 1=H, 2=V
	SHORT frequency;          // Frequency in 100's of MHz
	SHORT spare1;             // Reserved
	SHORT datatype;   // Data type code 
	                  // 0 or 2: bytes, minv=128 (2 is standard SIR format)
					  // 1     : 2-byte integers, minv=32767
					  // 4     : 4-byte IEEE floating point
	SHORT inodata;    // Value representing No Data, data types 0,1,2
	SHORT imin, imax; // Minimum and maximum useful value from creator program
//	SHORT fnodata;    // Value representing No Data, data type 4
//	SHORT fnodata2;    // Value representing No Data, data type 4
//	SHORT fmin;
//	SHORT fmin2;
//	SHORT fmax; // Min and max useful value from creator program
//	SHORT fmax2; // Min and max useful value from creator program
	char fnodata[4];
	char fmin[4], fmax[4];
//	FLOAT fmin, fmax;
	
	char type[138];   // 
//	SHORT null20;       // Should always == 0
	SHORT null2;       // Should always == 0
	SHORT null3;       // Should always == 0
	char title[80];   //
	SHORT null4;
	char tag[40];
	SHORT null5;
	char crproc[100];
	SHORT null6;
	char crtime[28];
	SHORT null7;

};

/*
struct sirheader
{
	SHORT xpixels, ypixels;   // Image size in pixels
	float xdegrees, ydegrees; // Span of degrees (in hundredths of degrees)
	SHORT headertype;         // Old header < 15
	float xscale, yscale;     // Scaling factors
	float olon, olat;  // Origin of longitude (-180<=olon<=180) and lattitude
	SHORT ioff;               // Offset to be added to scaled value
	SHORT iscale;             // Scale factor: ival=(val-ioff)/iscale
	SHORT year;               // Year of data used
	SHORT sday, smin;// Starting Julian Date, time of day for first data (in min)
	SHORT eday, emin;// Ending Julian Date, time of day for last data (in min)
	SHORT proj_type; // Projection type.  See DEFINEs above for types
	SHORT region;             // Region ID code
	SHORT image_type;         // Image type. See DEFINEs above for types
	SHORT null1;              // Should always == 0
	SHORT num_headers;        // Number of 512 byte header blocks
	SHORT num_desc;           // Number of 512 byte description blocks
	SHORT len_desc;           // Number of bytes in description
	SHORT num_opts;           // Number of optional integers
	SHORT polarization;       // 0=n/a, 1=H, 2=V
	SHORT frequency;          // Frequency in 100's of MHz
	SHORT spare1;             // Reserved
	SHORT datatype;   // Data type code 
	                  // 0 or 2: bytes, minv=128 (2 is standard SIR format)
					  // 1     : 2-byte integers, minv=32767
					  // 4     : 4-byte IEEE floating point
	float nodata;    // Value representing No Data, data types 0,1,2
	float v_min, v_max;
//	char *sensor, *title, *type, *tag, *crproc, *crtime, *description;	
	char sensor[42], title[82], type[140];
	char tag[42], crproc[102], crtime[30], *description;	
};
*/

struct sirheader
{
	int xpixels, ypixels;   // Image size in pixels
	float xdegrees, ydegrees; // Span of degrees (in hundredths of degrees)
	int headertype;         // Old header < 15
	float xscale, yscale;     // Scaling factors
	float olon, olat;  // Origin of longitude (-180<=olon<=180) and lattitude
	int ioff;               // Offset to be added to scaled value
	int iscale;             // Scale factor: ival=(val-ioff)/iscale
	int year;               // Year of data used
	int sday, smin;// Starting Julian Date, time of day for first data (in min)
	int eday, emin;// Ending Julian Date, time of day for last data (in min)
	int proj_type; // Projection type.  See DEFINEs above for types
	int region;             // Region ID code
	int image_type;         // Image type. See DEFINEs above for types
	int null1;              // Should always == 0
	int num_headers;        // Number of 512 byte header blocks
	int num_desc;           // Number of 512 byte description blocks
	int len_desc;           // Number of bytes in description
	int num_opts;           // Number of optional integers
	int polarization;       // 0=n/a, 1=H, 2=V
	int frequency;          // Frequency in 100's of MHz
	int spare1;             // Reserved
	int datatype;   // Data type code 
	                  // 0 or 2: bytes, minv=128 (2 is standard SIR format)
					  // 1     : 2-byte integers, minv=32767
					  // 4     : 4-byte IEEE floating point
	float nodata;    // Value representing No Data, data types 0,1,2
	float v_min, v_max;
//	char *sensor, *title, *type, *tag, *crproc, *crtime, *description;	
	char sensor[42], title[82], type[140];
	char tag[42], crproc[102], crtime[30], *description;	
};





/*
extern int read_sir_header(FILE *imf, int *nhead, int *ndes, int *nhtype,
		    int *idatatype, int *nsx, int *nsy, 
		    float *xdeg, float *ydeg, 
		    float *ascale, float *bscale, float *a0, float *b0, 
		    int *ioff, int *iscale, int *iyear, 
		    int *isday, int *ismin, int *ieday, int *iemin, 
		    int *iregion, int *itype, int *iopt,
		    int *ispare1, int *ispare2, int *ispare3,
		    float *anodata, float *v_min, float *v_max,
		    char *sensor, char *title, char *type, char *tag,
		    char *crproc, char *crtime, int maxdes, 
		    char *descrip, int *ldes, int maxi, short *iaopt,
		    int *nia);
*/

extern void print_head(int nhead, int ndes, int nhtype, int idatatype, int nsx,
		int nsy, float xdeg, float ydeg, float ascale,
		float bscale, float a0, float b0, 
		int ioff, int iscale, int iyear, int isday, int ismin,
		int ieday, int iemin, int iregion, int itype, int iopt,
		int ipol, int ifreqhm, int ispare1,
		float anodata, float v_min, float v_max,
		char *sensor, char *title, char *type, char *tag, char
		*crproc, char *crtime, 
		char *descrip, int ldes, short *iaopt, int nia);

extern int write_sir_header(FILE *imf, char *title, char *anont,
		int nsx, int nsy, float xdeg, float ydeg, 
		float ascale, float bscale, float a0, float b0, 
		int ioff, int iscale, int iyear, int isday, int ismin,
		int ieday, int iemin, int iregion, int itype, int iopt,
		short iaopt[128]);


extern int putsir(char *fname, char *title, char *anont, int nsx, 
		int nsy, float xdeg, float ydeg, float ascale, float bscale,
		float a0, float b0, float *stval, int ioff, int iscale,
		int iyear, int isday, int ismin, int ieday, int iemin, 
		int iregion, int itype, int iopt, short iaopt[128], int *ierr);

extern void print_sir_header(struct sirheader h);
extern int read_sir_header(FILE *imf, struct sirheader *h);
extern int read_sir_data(FILE *imf, float *data);
extern float *LoadSIR(char *filename, struct sirheader *h);
extern int match_topography(char *insir, char *topo, char *outsir);
extern int match_topography_from_globe(char *insir, char *globefile, char *outsir);

#endif
